@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import contextlib
 import logging
 import time
 import warnings
@@ -8,7 +9,7 @@ from uuid import uuid4
 
 import attr
 import tablib
-from sqlalchemy import alias, func
+from sqlalchemy import alias, func, select
 from sureberus import normalize_dict, normalize_schema
 
 from recipe.dynamic_extensions import run_hooks
@@ -75,9 +76,12 @@ class Recipe(object):
     ):
         self._id = str(uuid4())[:8]
         self._query = None
+        self._select = None
         self._all = None
+        self._total_count = None
 
         self._select_from = None
+        self._allow_multiple_tables = False
         self.shelf(shelf)
 
         # Stores all ingredients used in the recipe
@@ -125,6 +129,7 @@ class Recipe(object):
 
             query: An optional SQLAlchemy query to calculate total_count for.
               If None, the recipe query will be used.
+              If a query is passed, no caching will be done.
 
         Returns:
             A count of the number of rows that are returned by this query.
@@ -132,29 +137,35 @@ class Recipe(object):
         if query is None:
             query = self.query()
 
-        count_query = self._session.query(func.count().label("count")).select_from(
-            query.limit(None).offset(None).order_by(None).subquery()
-        )
-
-        # If recipe_caching is installed, apply caching to this query.
-        try:
-            from recipe_caching.mappers import FromCache
-
-            count_query = count_query.options(
-                FromCache(self._cache_region, cache_prefix=self._cache_prefix)
+        if self._total_count is None or query is not None:
+            count_query = self._session.query(func.count().label("count")).select_from(
+                query.limit(None).offset(None).order_by(None).subquery()
             )
-        except ImportError:
-            pass
 
-        return count_query.scalar()
+            # If recipe_caching is installed, apply caching to this query.
+            try:
+                from recipe_caching.mappers import FromCache
+
+                count_query = count_query.options(
+                    FromCache(self._cache_region, cache_prefix=self._cache_prefix)
+                )
+            except ImportError:
+                pass
+
+            count = count_query.scalar()
+            if query is not None:
+                return count
+            self._total_count = count
+        return self._total_count
 
     def reset(self):
         self._query = None
         self._all = None
+        self._total_count = None
         return self
 
     @classmethod
-    def from_config(cls, shelf, obj, **kwargs):
+    def from_config(cls, shelf, spec, **kwargs):
         """
         Construct a Recipe from a plain Python dictionary.
 
@@ -173,13 +184,14 @@ class Recipe(object):
                     new[k] = d[k]
             return new
 
-        core_kwargs = subdict(obj, recipe_schema["schema"].keys())
+        core_kwargs = subdict(spec, recipe_schema["schema"].keys())
         core_kwargs = normalize_schema(recipe_schema, core_kwargs)
         core_kwargs["filters"] = [
+            # FIXME: This doesn't exist
             parse_unvalidated_condition(filter, shelf.Meta.select_from)
             if isinstance(filter, dict)
             else filter
-            for filter in obj.get("filters", [])
+            for filter in spec.get("filters", [])
         ]
         core_kwargs.update(kwargs)
         recipe = cls(shelf=shelf, **core_kwargs)
@@ -188,7 +200,7 @@ class Recipe(object):
         for ext in recipe.recipe_extensions:
             additional_schema = getattr(ext, "recipe_schema", None)
             if additional_schema is not None:
-                ext_data = subdict(obj, additional_schema.keys())
+                ext_data = subdict(spec, additional_schema.keys())
                 ext_data = normalize_dict(additional_schema, ext_data)
                 recipe = ext.from_config(ext_data)
         return recipe
@@ -205,24 +217,17 @@ class Recipe(object):
         :param name:
         :return:
         """
-        try:
+        with contextlib.suppress(AttributeError):
             return self.__getattribute__(name)
-        except AttributeError:
-            pass
-
         for extension in self.recipe_extensions:
-            try:
+            with contextlib.suppress(AttributeError):
                 proxy_callable = getattr(extension, name)
                 break
-            except AttributeError:
-                pass
-
         try:
             proxy_callable
         except NameError:
             raise AttributeError(
-                "{} isn't available on this recipe, "
-                "you may need to add an extension".format(name)
+                f"{name} isn't available on this recipe, you may need to add an extension"
             )
 
         return proxy_callable
@@ -246,17 +251,21 @@ class Recipe(object):
         self._cache_region = value
 
     @recipe_arg()
-    def cache_prefix(self, value) -> Recipe:
+    def cache_prefix(self, value: str) -> Recipe:
         """Set a cache prefix for recipe-caching to use"""
         assert isinstance(value, str)
         self._cache_prefix = value
 
     @recipe_arg()
-    def use_cache(self, value) -> Recipe:
+    def use_cache(self, value: bool) -> Recipe:
         """If False, invalidate the cache before fetching data."""
         assert isinstance(value, bool)
         self._use_cache = value
 
+    @recipe_arg()
+    def allow_multiple_tables(self, value: bool) -> Recipe:
+        self._allow_multiple_tables = value
+
     @recipe_arg()
     def shelf(self, shelf=None) -> Recipe:
         """Defines a shelf to use for this recipe"""
@@ -383,23 +392,57 @@ class Recipe(object):
 
     def _is_postgres(self):
         """Determine if the running engine is postgres"""
-        try:
+        with contextlib.suppress(Exception):
             driver = self._session.bind.url.drivername
             if "redshift" in driver or "postg" in driver or "pg" in driver:
                 return True
-        except:
-            pass
         return False
 
     def _is_redshift(self):
-        try:
+        with contextlib.suppress(Exception):
             driver = self._session.bind.url.drivername
             if "redshift" in driver:
                 return True
-        except:
-            pass
         return False
 
+    def select(self):
+        """
+        Generate a SQLALchemy core select.
+
+        This is a lighter way to generate queries. Extensions are
+        not currently supported.
+        """
+        if self._select is not None:
+            return self._select
+
+        if hasattr(self, "optimize_redshift"):
+            self.optimize_redshift(self._is_redshift())
+
+        if len(self._cauldron.ingredients()) == 0:
+            raise BadRecipe("No ingredients have been added to this recipe")
+
+        # Step 1: Gather up global filters and user filters and
+        # apply them as if they had been added to recipe().filters(...)
+        for extension in self.recipe_extensions:
+            extension.add_ingredients()
+
+        select_parts = self._cauldron.brew_select_parts(self._order_bys)
+        if self._select_from is not None:
+            sel = select(select_parts.columns[:1]).select_from(self._select_from)
+        else:
+            sel = select(select_parts.columns[:1])
+        if select_parts.group_bys:
+            sel = sel.group_by(*select_parts.group_bys)
+        if select_parts.order_bys:
+            sel = sel.order_by(*select_parts.order_bys)
+        if select_parts.filters:
+            sel = sel.where(*select_parts.filters)
+        if select_parts.havings:
+            sel = sel.having(*select_parts.havings)
+
+        self._select = sel
+        return self._select
+
     def query(self):
         """
         Generates a query using the ingredients supplied by the recipe.
@@ -451,17 +494,14 @@ class Recipe(object):
             for having in recipe_parts["havings"]:
                 recipe_parts["query"] = recipe_parts["query"].having(having)
 
-        for extension in self.recipe_extensions:
-            recipe_parts = extension.modify_prequery_parts(recipe_parts)
-
         if (
-            self._select_from is None
+            self._allow_multiple_tables is False
+            and self._select_from is None
             and len(recipe_parts["query"].selectable.froms) != 1
         ):
             raise BadRecipe(
-                "Recipes must use ingredients that all come from "
-                "the same table. \nDetails on this recipe:\n{"
-                "}".format(str(self._cauldron))
+                f"Recipes must use ingredients that all come from the same table. \n"
+                f"Details on this recipe:\n{str(self._cauldron)}"
             )
 
         for extension in self.recipe_extensions:
@@ -490,8 +530,7 @@ class Recipe(object):
         """A convenience method to determine the table the query is
         selecting from
         """
-        descriptions = self.query().column_descriptions
-        if descriptions:
+        if descriptions := self.query().column_descriptions:
             return descriptions[0]["entity"]
         else:
             return None
@@ -509,9 +548,7 @@ class Recipe(object):
 
     def as_table(self, name=None):
         """Return an alias to a table"""
-        if name is None:
-            name = self._id
-        return alias(self.subquery(), name=name)
+        return alias(self.subquery(), name=name or self._id)
 
     def all(self):
         """Return a (potentially cached) list of result objects."""
@@ -520,11 +557,8 @@ class Recipe(object):
 
         if self._all is None:
             fetchtime = time.time()
-            if not self._use_cache:
-                # Invalidate this query in the cache
-                # to ensure it gets fresh data from the database
-                if hasattr(self._query, "invalidate"):
-                    self._query.invalidate()
+            if not self._use_cache and hasattr(self._query, "invalidate"):
+                self._query.invalidate()
 
             self._all = self._cauldron.enchant(
                 self._query.all(), cache_context=self.cache_context
@@ -544,10 +578,7 @@ class Recipe(object):
     def one(self):
         """Return the first element on the result"""
         all = self.all()
-        if len(all) > 0:
-            return all[0]
-        else:
-            return []
+        return all[0] if len(all) > 0 else []
 
     def first(self):
         """Return the first element on the result"""
