@@ -1,7 +1,6 @@
-import attr
 from functools import total_ordering
 from uuid import uuid4
-from sqlalchemy import Float, String, and_, between, case, cast, func, or_, text
+from sqlalchemy import Float, String, and_, between, case, cast, func, or_, text, not_
 from recipe.exceptions import BadIngredient
 from recipe.utils import AttrDict, filter_to_string
 from recipe.utils.datatype import (
@@ -10,6 +9,7 @@ from recipe.utils.datatype import (
     determine_datatype,
     datatype_from_column_expression,
 )
+from typing import List
 
 ALLOWED_OPERATORS = set(
     [
@@ -31,6 +31,15 @@ ALLOWED_OPERATORS = set(
 )
 
 
+def is_nested_condition(v) -> bool:
+    return isinstance(v, dict) and "operator" in v and "value" in v
+
+
+def contains_complex_values(v: List) -> bool:
+    """Check if any of the values in a list requires special handling to filter on"""
+    return None in v or any(map(is_nested_condition, v))
+
+
 @total_ordering
 class Ingredient(object):
     """Ingredients combine to make a SQLAlchemy query.
@@ -173,17 +182,14 @@ class Ingredient(object):
             return ()
 
         elif len(self.columns) == 1:
-            if self.formatters:
-                return ("_raw",)
-            else:
-                return ("",)
+            return ("_raw",) if self.formatters else ("",)
         else:
             raise BadIngredient(
                 "column_suffixes must be supplied if there is more than one column"
             )
 
     @property
-    def query_columns(self):
+    def labeled_columns(self):
         """Yield labeled columns to be used as a select in a query."""
         self._labels = []
         for column, suffix in zip(self.columns, self.make_column_suffixes()):
@@ -200,7 +206,7 @@ class Ingredient(object):
         """
         # Ensure the labels are generated
         if not self._labels:
-            list(self.query_columns)
+            list(self.labeled_columns)
 
         if self.group_by_strategy == "labels":
             if self.ordering == "desc":
@@ -363,37 +369,40 @@ class Ingredient(object):
         elif datatype == "datetime":
             value = list(map(convert_datetime, value))
 
-        if operator == "in":
-            # Default operator is 'in' so if no operator is provided, handle
-            # like an 'in'
-            if None in value:
-                # filter out the Nones
-                non_none_value = sorted([v for v in value if v is not None])
-                if non_none_value:
-                    return or_(
-                        filter_column.is_(None), filter_column.in_(non_none_value)
-                    )
-                else:
-                    return filter_column.is_(None)
-            else:
-                # Sort to generate deterministic query sql for caching
-                value = sorted(value)
-                return filter_column.in_(value)
+        if operator == "and":
+            conditions = [self.build_filter(x["value"], x["operator"]) for x in value]
+            return and_(*conditions)
+
+        if operator in ("in", "notin"):
+            if contains_complex_values(value):
+                # A list may contain additional operators or nones
+                # Convert from:
+                # department__in: [None, "A", 'B"]
+                #
+                # to the SQL
+                #
+                # department in ("A", "B") OR department is null
+                simple_values = sorted(
+                    [v for v in value if v is not None and not is_nested_condition(v)]
+                )
+                nested_conditions = [v for v in value if is_nested_condition(v)]
+                conditions = []
+                if None in value:
+                    conditions.append(filter_column.is_(None))
+                if simple_values:
+                    conditions.append(filter_column.in_(simple_values))
+                conditions.extend(
+                    self.build_filter(cond["value"], operator=cond["operator"])
+                    for cond in nested_conditions
+                )
+                cond = or_(*conditions)
 
-        elif operator == "notin":
-            if None in value:
-                # filter out the Nones
-                non_none_value = sorted([v for v in value if v is not None])
-                if non_none_value:
-                    return and_(
-                        filter_column.isnot(None), filter_column.notin_(non_none_value)
-                    )
-                else:
-                    return filter_column.isnot(None)
             else:
                 # Sort to generate deterministic query sql for caching
-                value = sorted(value)
-                return filter_column.notin_(value)
+                cond = filter_column.in_(sorted(value))
+
+            return not_(cond) if operator == "notin" else cond
+
         elif operator == "between":
             if len(value) != 2:
                 ValueError(
@@ -630,7 +639,7 @@ class Dimension(Ingredient):
     def group_by(self):
         # Ensure the labels are generated
         if not self._labels:
-            list(self.query_columns)
+            list(self.labeled_columns)
 
         if self.group_by_strategy == "labels":
             return [lbl for _, lbl in zip(self._group_by, self._labels)]
@@ -647,10 +656,9 @@ class Dimension(Ingredient):
         a row
         """
         # This will format the value field
-        for extra in super(Dimension, self).cauldron_extras:
-            yield extra
-
-        yield self.id + "_id", lambda row: getattr(row, self.id_prop)
+        yield from super(Dimension, self).cauldron_extras
+        if "id" not in self.role_keys:
+            yield (f"{self.id}_id", lambda row: getattr(row, self.id_prop))
 
     def make_column_suffixes(self):
         """Make sure we have the right column suffixes. These will be appended
@@ -662,20 +670,19 @@ class Dimension(Ingredient):
             value_suffix = ""
 
         return tuple(
-            value_suffix if role == "value" else "_" + role for role in self.role_keys
+            value_suffix if role == "value" else f"_{role}" for role in self.role_keys
         )
 
     @property
     def id_prop(self):
         """The label of this dimensions id in the query columns"""
         if "id" in self.role_keys:
-            return self.id + "_id"
-        else:
+            return f"{self.id}_id"
             # Use the value dimension
-            if self.formatters:
-                return self.id + "_raw"
-            else:
-                return self.id
+        if self.formatters:
+            return f"{self.id}_raw"
+        else:
+            return self.id
 
 
 class IdValueDimension(Dimension):
