@@ -3,16 +3,17 @@
 import time
 
 from freezegun import freeze_time
-from sqlalchemy import Table, Column, String, Integer
-from sqlalchemy.ext.serializer import dumps, loads
+from lark import GrammarError
+from sqlalchemy import Column, Integer, String, Table
 from sqlalchemy.ext.declarative import declarative_base
+from sqlalchemy.ext.serializer import dumps, loads
 
+from recipe.schemas.builders import SQLAlchemyBuilder
 from recipe.schemas.expression_grammar import (
-    SQLAlchemyBuilder,
-    is_valid_column,
-    make_columns_for_selectable,
-    make_columns_grammar,
     gather_columns,
+    is_valid_column,
+    make_column_collection_for_selectable,
+    make_columns_grammar,
 )
 from recipe.utils.formatting import expr_to_str
 from tests.test_base import RecipeTestCase
@@ -55,8 +56,10 @@ class BuildGrammarTestCase(RecipeTestCase):
             DateTesterData,
         ]
 
-    def assertSelectableGrammar(self, selectable, grammar_text: str):
-        grammar = make_columns_grammar(make_columns_for_selectable(selectable))
+    def assertSelectableGrammar(self, selectable, grammar_text: str, *, namespace=None):
+        grammar = make_columns_grammar(
+            make_column_collection_for_selectable(selectable, namespace=namespace)
+        )
 
         if str_dedent(grammar) != str_dedent(grammar_text):
             print(
@@ -66,24 +69,25 @@ class BuildGrammarTestCase(RecipeTestCase):
 
     def test_make_columns_for_table(self):
         expected_column_keys = [
-            ["date_0", "datetime_0", "num_0", "str_0", "str_1"],
-            ["bool_0", "date_0", "datetime_0", "num_0", "str_0", "str_1", "str_2"],
-            ["num_0", "str_0", "str_1"],
-            ["num_0", "str_0", "str_1"],
-            ["date_0", "datetime_0", "num_0", "str_0", "str_1"],
-            ["num_0"],
+            ["date", "datetime", "num", "str", "str"],
+            ["bool", "date", "datetime", "num", "str", "str", "str"],
+            ["num", "str", "str"],
+            ["num", "str", "str"],
+            ["date", "datetime", "num", "str", "str"],
+            ["num"],
         ]
 
         for selectable, expected_column_keys in zip(
             self.selectables, expected_column_keys
         ):
-            c = make_columns_for_selectable(selectable)
-            self.assertEqual(sorted(list(c.keys())), expected_column_keys)
+            cc = make_column_collection_for_selectable(selectable)
+            ctypes = sorted([col.datatype for col in cc.columns])
+            self.assertEqual(ctypes, expected_column_keys)
 
         with self.assertRaises(Exception):
-            make_columns_for_selectable(None)
+            make_column_collection_for_selectable(None)
         with self.assertRaises(Exception):
-            make_columns_for_selectable("foo")
+            make_column_collection_for_selectable("foo")
 
     def test_make_columns_grammar(self):
         expected_grammars = [
@@ -99,9 +103,9 @@ class BuildGrammarTestCase(RecipeTestCase):
             date_0: "[" + /test_date/i + "]" | /test_date/i
             datetime_0: "[" + /test_datetime/i + "]" | /test_datetime/i
             num_0: "[" + /score/i + "]" | /score/i
-            str_0: "[" + /username/i + "]" | /username/i
-            str_1: "[" + /department/i + "]" | /department/i
-            str_2: "[" + /testid/i + "]" | /testid/i
+            str_0: "[" + /department/i + "]" | /department/i
+            str_1: "[" + /testid/i + "]" | /testid/i
+            str_2: "[" + /username/i + "]" | /username/i
             """,
             """
             num_0: "[" + /age/i + "]" | /age/i
@@ -110,8 +114,8 @@ class BuildGrammarTestCase(RecipeTestCase):
             """,
             """
             num_0: "[" + /age/i + "]" | /age/i
-            str_0: "[" + /firstlast_id/i + "]" | /firstlast_id/i
-            str_1: "[" + /firstlast/i + "]" | /firstlast/i
+            str_0: "[" + /firstlast/i + "]" | /firstlast/i
+            str_1: "[" + /firstlast_id/i + "]" | /firstlast_id/i
             """,
             """
             date_0: "[" + /birth_date/i + "]" | /birth_date/i
@@ -127,6 +131,20 @@ class BuildGrammarTestCase(RecipeTestCase):
         for selectable, expected_grammar in zip(self.selectables, expected_grammars):
             self.assertSelectableGrammar(selectable, expected_grammar)
 
+    def test_make_columns_grammar_with_namespace(self):
+        # If we pass a namespace, field names will be prefixed by the namespace
+        expected_grammars = [
+            """
+            date_0: "[" + /foo\.birth_date/i + "]" | /foo\.birth_date/i
+            datetime_0: "[" + /foo\.dt/i + "]" | /foo\.dt/i
+            num_0: "[" + /foo\.age/i + "]" | /foo\.age/i
+            str_0: "[" + /foo\.first/i + "]" | /foo\.first/i
+            str_1: "[" + /foo\.last/i + "]" | /foo\.last/i
+            """
+        ]
+        for selectable, expected_grammar in zip(self.selectables, expected_grammars):
+            self.assertSelectableGrammar(selectable, expected_grammar, namespace="foo")
+
     def test_gather_columns(self):
         """Gathered columns collects all the rules for column types into a single rule"""
         expected_gathered_columns = [
@@ -188,15 +206,15 @@ class BuildGrammarTestCase(RecipeTestCase):
         for selectable, expected_gathered in zip(
             self.selectables, expected_gathered_columns
         ):
-            columns = make_columns_for_selectable(selectable)
+            columns = make_column_collection_for_selectable(selectable)
             gathered_columns = f"""
             {gather_columns("unusable_col", columns, "unusable")}
-            {gather_columns("date.1", columns, "date", ["extra_date_rule"])}
-            {gather_columns("datetime.2", columns, "datetime", ["extra_datetime_rule"])}
-            {gather_columns("datetime_end.1", columns, "datetime", ["datetime_end_conv", "datetime_aggr"])}
-            {gather_columns("boolean.1", columns, "bool", ["TRUE", "FALSE", "extra_bool_rule"])}
-            {gather_columns("string.1", columns, "str", ["ESCAPED_STRING", "extra_string_rule"])}
-            {gather_columns("num.1", columns, "num", ["NUMBER", "extra_num_rule"])}
+            {gather_columns("date.1", columns, "date", additional_rules=["extra_date_rule"])}
+            {gather_columns("datetime.2", columns, "datetime", additional_rules=["extra_datetime_rule"])}
+            {gather_columns("datetime_end.1", columns, "datetime", additional_rules=["datetime_end_conv", "datetime_aggr"])}
+            {gather_columns("boolean.1", columns, "bool", additional_rules=["TRUE", "FALSE", "extra_bool_rule"])}
+            {gather_columns("string.1", columns, "str", additional_rules=["ESCAPED_STRING", "extra_string_rule"])}
+            {gather_columns("num.1", columns, "num", additional_rules=["NUMBER", "extra_num_rule"])}
             """
             self.assertEqual(
                 str_dedent(gathered_columns), str_dedent(expected_gathered)
@@ -208,7 +226,10 @@ class GrammarTestCase(RecipeTestCase):
 
     def setUp(self):
         super().setUp()
-        self.builder = SQLAlchemyBuilder.get_builder(self.datatypes_table)
+        extra_selectables = [(self.scores_table, "scores")]
+        self.builder = SQLAlchemyBuilder.get_builder(
+            self.datatypes_table, extra_selectables=extra_selectables
+        )
 
     def examples(self, input_rows):
         """Take input where each line looks like
@@ -270,6 +291,8 @@ class TestSQLAlchemyBuilder(GrammarTestCase):
         [ScORE] + [ScORE]               -> sum(datatypes.score + datatypes.score)
         max([ScORE] + [ScORE])          -> max(datatypes.score + datatypes.score)
         max(score) - min(score)         -> max(datatypes.score) - min(datatypes.score)
+        max(scores.score)               -> max(scores.score)
+        max([score] - [scores.score])   -> max(datatypes.score - scores.score)
         """
 
         for field, expected_sql in self.examples(good_examples):
@@ -291,12 +314,29 @@ class TestSQLAlchemyBuilder(GrammarTestCase):
         min(department)                 -> str
         min(test_date)                  -> date
         count(*)                        -> num
+        count(department > "foo")       -> num
         substr(department, 5)           -> str
         substr(department, 5, 5)        -> str
+        max([score] - [scores.score])   -> num
         """
 
         for field, expected_data_type in self.examples(good_examples):
-            _, data_type = self.builder.parse(field)
+            _, data_type = self.builder.parse(field, debug=True)
+            self.assertIs(type(data_type), str)
+            self.assertEqual(data_type, expected_data_type)
+
+    def test_disallow_literals(self):
+        examples = """
+        "22"          -> str
+        2.0           -> num
+        2.0 + 1.0     -> num
+        "220" + "foo" -> str
+        5             -> num
+        """
+
+        for field, expected_data_type in self.examples(examples):
+            _, data_type = self.builder.parse(field, debug=True)
+            self.assertIs(type(data_type), str)
             self.assertEqual(data_type, expected_data_type)
 
     def test_selectable_recipe(self):
@@ -588,6 +628,8 @@ class TestDataTypesTable(GrammarTestCase):
         [score] between [score] and [score]                   -> datatypes.score BETWEEN datatypes.score AND datatypes.score
         [username] between "a" and "z"                        -> datatypes.username BETWEEN 'a' AND 'z'
         [username] between [department] and "z"               -> datatypes.username BETWEEN datatypes.department AND 'z'
+        count_distinct([score] > 80)                          -> count(DISTINCT (datatypes.score > 80))
+        count([score] > 80)                                   -> count(datatypes.score > 80)
         """
 
         for field, expected_sql in self.examples(good_examples):
@@ -842,13 +884,13 @@ class TestDataTypesTableDatesInBigquery(TestDataTypesTableDates):
         """bigquery generates different sql for date conversions"""
         good_examples = f"""
         month([test_date]) > date("2020-12-30")          -> date_trunc(datatypes.test_date, month) > '2020-12-30'
-        month([test_datetime]) > date("2020-12-30")      -> timestamp_trunc(datatypes.test_datetime, month) > '2020-12-30'
-        date("2020-12-30") < month([test_datetime])      -> timestamp_trunc(datatypes.test_datetime, month) > '2020-12-30'
+        month([test_datetime]) > date("2020-12-30")      -> datetime(timestamp_trunc(datatypes.test_datetime, month)) > '2020-12-30'
+        date("2020-12-30") < month([test_datetime])      -> datetime(timestamp_trunc(datatypes.test_datetime, month)) > '2020-12-30'
         day([test_date]) > date("2020-12-30")            -> date_trunc(datatypes.test_date, day) > '2020-12-30'
         week([test_date]) > date("2020-12-30")           -> date_trunc(datatypes.test_date, week(monday)) > '2020-12-30'
         quarter([test_date]) > date("2020-12-30")        -> date_trunc(datatypes.test_date, quarter) > '2020-12-30'
         year([test_date]) > date("2020-12-30")           -> date_trunc(datatypes.test_date, year) > '2020-12-30'
-        date([test_datetime])                            -> timestamp_trunc(datatypes.test_datetime, day)
+        date([test_datetime])                            -> datetime(timestamp_trunc(datatypes.test_datetime, day))
         """
 
         for field, expected_sql in self.examples(good_examples):
@@ -1096,7 +1138,7 @@ if(department = "foo", department, valid_score, score)
                 self.builder.parse(field, forbid_aggregation=True, debug=True)
             if str(e.exception).strip() != expected_error.strip():
                 print("===actual===")
-                print(str(e.exception))
+                print(e.exception)
                 print("===expected===")
                 print(expected_error)
                 print("===" * 10)
