@@ -1,10 +1,7 @@
 from json import dumps
 from copy import copy
-from csv import DictReader
-
-import pytest
 from faker import Faker
-from sqlalchemy import func
+from sqlalchemy import func, and_, or_
 from sureberus.errors import BadType, SureError
 
 from recipe import BadRecipe, Dimension, Metric, Recipe, Shelf
@@ -18,6 +15,7 @@ from recipe.extensions import (
     RecipeExtension,
     SummarizeOver,
     handle_directives,
+    is_compound_filter,
 )
 from recipe.utils import generate_faker_seed, recipe_arg
 from tests.test_base import RecipeTestCase
@@ -175,7 +173,37 @@ class AutomaticFiltersTestCase(RecipeTestCase):
             },
         ):
             ext = recipe.recipe_extensions[0]
-            self.assertEqual(ext._automatic_filters, {"first": ["foo"]})
+            self.assertEqual(ext._automatic_filters, [{"first": ["foo"]}])
+            self.assertEqual(ext.include_keys, ("foo", "moo"))
+            self.assertEqual(ext.exclude_keys, ("foo", "you"))
+            self.assertFalse(ext.apply)
+
+    def test_multiple_filter_dicts(self):
+        """Check the internal state of an extension after configuration"""
+        for recipe in self.recipe_list(
+            (
+                self.recipe()
+                .metrics("age")
+                .dimensions("first")
+                .automatic_filters({"first": ["foo"]})
+                .automatic_filters({"second": ["moo"]})
+                .include_automatic_filter_keys("foo", "moo")
+                .exclude_automatic_filter_keys("foo", "you")
+                .apply_automatic_filters(False)
+            ),
+            {
+                "metrics": ["age"],
+                "dimensions": ["first"],
+                "automatic_filters": [{"first": ["foo"]}, {"second": ["moo"]}],
+                "include_automatic_filter_keys": ["foo", "moo"],
+                "exclude_automatic_filter_keys": ["foo", "you"],
+                "apply_automatic_filters": False,
+            },
+        ):
+            ext = recipe.recipe_extensions[0]
+            self.assertEqual(
+                ext._automatic_filters, [{"first": ["foo"]}, {"second": ["moo"]}]
+            )
             self.assertEqual(ext.include_keys, ("foo", "moo"))
             self.assertEqual(ext.exclude_keys, ("foo", "you"))
             self.assertFalse(ext.apply)
@@ -316,6 +344,56 @@ class AutomaticFiltersTestCase(RecipeTestCase):
             GROUP BY first""",
             )
 
+    def test_multiple_automatic_filters(self):
+        """Automatic filters can be passed multiple times and all will apply"""
+        for recipe in self.recipe_list(
+            {
+                "metrics": ["age"],
+                "dimensions": ["first"],
+                "automatic_filters": [{"first": ["foo"]}, {"last__gt": "x"}],
+            }
+        ):
+            self.assertTrue(recipe.recipe_extensions[0].apply)
+            self.assertRecipeSQLContains(recipe, "foo.first IN ('foo')")
+            self.assertRecipeSQLContains(recipe, "foo.last > 'x'")
+
+        recipe = (
+            self.recipe()
+            .metrics("age")
+            .dimensions("first")
+            .automatic_filters({"first": ["foo"]})
+            .automatic_filters({"last__gt": "t"})
+        )
+        self.assertTrue(recipe.recipe_extensions[0].apply)
+        self.assertRecipeSQLContains(recipe, "foo.first IN ('foo')")
+        self.assertRecipeSQLContains(recipe, "foo.last > 't'")
+
+    def test_multiple_automatic_filters_with_exclude(self):
+        """Automatic filters can be passed multiple times and all will apply"""
+        for recipe in self.recipe_list(
+            {
+                "metrics": ["age"],
+                "dimensions": ["first"],
+                "automatic_filters": [{"first": ["foo"]}, {"last__gt": "x"}],
+                "exclude_automatic_filter_keys": ["last"],
+            }
+        ):
+            self.assertTrue(recipe.recipe_extensions[0].apply)
+            self.assertRecipeSQLContains(recipe, "foo.first IN ('foo')")
+            self.assertRecipeSQLNotContains(recipe, "foo.last > 'x'")
+
+        recipe = (
+            self.recipe()
+            .metrics("age")
+            .dimensions("first")
+            .automatic_filters({"first": ["foo"]})
+            .automatic_filters({"last__gt": "t"})
+            .exclude_automatic_filter_keys("last")
+        )
+        self.assertTrue(recipe.recipe_extensions[0].apply)
+        self.assertRecipeSQLContains(recipe, "foo.first IN ('foo')")
+        self.assertRecipeSQLNotContains(recipe, "foo.last > 't'")
+
     def test_automatic_filters_with_unknown_dim(self):
         """Test filters built using an unknown dimension"""
         for recipe in self.recipe_list(
@@ -411,6 +489,16 @@ class AutomaticFiltersTestCase(RecipeTestCase):
                 GROUP BY first""",
             )
 
+    def test_is_compound_filter(self):
+        valid_compound_keys = ["a,b", "a,b,c", "a,b,,c", ",a,bb,aac", ","]
+        for k in valid_compound_keys:
+            self.assertTrue(is_compound_filter(k))
+
+        # These are not compound filters
+        invalid_compound_keys = ["ab", "", "ab__notin"]
+        for k in invalid_compound_keys:
+            self.assertFalse(is_compound_filter(k))
+
     def test_operators_and_compound_filters(self):
         """Test operators and compound filters. Filter values may be json encoded"""
         # Testing operators
@@ -461,6 +549,12 @@ class AutomaticFiltersTestCase(RecipeTestCase):
   OR foo.first = 'chicken'
   AND foo.last = 'cluck'""",
             ),
+            (
+                {"first__in,first__notin": ['[["foo"], ["moo","cow"]]']},
+                """foo.first IN ('foo')
+  AND foo.first NOT IN ('cow',
+                        'moo')""",
+            ),
         )
         for af, expected_sql in values:
             # json encode all the values
@@ -1006,16 +1100,16 @@ class PaginateTestCase(RecipeTestCase):
         self.assertRecipeCSV(
             recipe,
             """
-            idvalue_state_id,idvalue_state,pop2000,idvalue_state_id
-            Tennessee,State:Tennessee,5685230,Tennessee
+            idvalue_state_id,idvalue_state,pop2000
+            Tennessee,State:Tennessee,5685230
             """,
         )
 
         self.assertRecipeCSV(
             recipe,
             """
-            idvalue_state_id,idvalue_state,pop2000,idvalue_state_id
-            Tennessee,State:Tennessee,5685230,Tennessee
+            idvalue_state_id,idvalue_state,pop2000
+            Tennessee,State:Tennessee,5685230
             """,
         )
 
