@@ -393,7 +393,7 @@ class TestIngredientBuildFilter(RecipeTestCase):
                 strdim,
                 ["moo", None],
                 "notin",
-                "foo.first IS NOT NULL AND foo.first NOT IN ('moo')",
+                "NOT (foo.first IS NULL OR foo.first IN ('moo'))",
             ),
             (strdim, [None, None], "notin", "foo.first IS NOT NULL"),
             # Between values are not sorted
@@ -423,6 +423,28 @@ class TestIngredientBuildFilter(RecipeTestCase):
                 "between",
                 "foo.dt BETWEEN '1970-01-03 00:00:00' AND '1970-01-06 00:00:00'",
             ),
+            # Nested filters
+            (
+                strdim,
+                ["moo", {"operator": "like", "value": "%o"}],
+                None,
+                "foo.first IN ('moo') OR foo.first LIKE '%o'",
+            ),
+            (
+                strdim,
+                ["moo", {"operator": "notin", "value": ["cow", "pig"]}, "horse"],
+                None,
+                "foo.first IN ('horse', 'moo') OR foo.first NOT IN ('cow', 'pig')",
+            ),
+            (
+                strdim,
+                [
+                    {"operator": "in", "value": [1, 2, 3]},
+                    {"operator": "notin", "value": [3, 4, 5]},
+                ],
+                "and",
+                "foo.first IN (1, 2, 3) AND foo.first NOT IN (3, 4, 5)",
+            ),
         ]
 
         baddata = [
@@ -636,9 +658,7 @@ class TestDimension(RecipeTestCase):
             id="moo",
         )
         extras = list(d.cauldron_extras)
-        self.assertEqual(len(extras), 1)
-        # id gets injected in the response
-        self.assertEqual(extras[0][0], "moo_id")
+        self.assertEqual(len(extras), 0)
         self.assertEqual(d.role_keys, ["id", "value", "age"])
         self.assertEqual(len(d.group_by), 3)
         self.assertEqual(len(d.columns), 3)
@@ -704,21 +724,18 @@ class IdValueDimensionTestCase(RecipeTestCase):
             self.basic_table.c.first, self.basic_table.c.last, id="moo"
         )
         extras = list(d.cauldron_extras)
-        self.assertEqual(len(extras), 1)
-        # id gets injected in the response
-        self.assertEqual(extras[0][0], "moo_id")
+        self.assertEqual(len(extras), 0)
 
         d = IdValueDimension(
             self.basic_table.c.first,
             self.basic_table.c.last,
             id="moo",
-            formatters=[lambda x: x + "moo"],
+            formatters=[lambda x: f"{x}moo"],
         )
         extras = list(d.cauldron_extras)
-        self.assertEqual(len(extras), 2)
+        self.assertEqual(len(extras), 1)
         # formatted value and id gets injected in the response
         self.assertEqual(extras[0][0], "moo")
-        self.assertEqual(extras[1][0], "moo_id")
 
     def test_dimension_roles_cauldron_extras(self):
         """Creating a dimension with roles performs the same as
@@ -727,21 +744,18 @@ class IdValueDimensionTestCase(RecipeTestCase):
             self.basic_table.c.first, id_expression=self.basic_table.c.last, id="moo"
         )
         extras = list(d.cauldron_extras)
-        self.assertEqual(len(extras), 1)
-        # id gets injected in the response
-        self.assertEqual(extras[0][0], "moo_id")
+        self.assertEqual(len(extras), 0)
 
         d = Dimension(
             self.basic_table.c.first,
             id_expression=self.basic_table.c.last,
             id="moo",
-            formatters=[lambda x: x + "moo"],
+            formatters=[lambda x: f"{x}moo"],
         )
         extras = list(d.cauldron_extras)
-        self.assertEqual(len(extras), 2)
+        self.assertEqual(len(extras), 1)
         # formatted value and id gets injected in the response
         self.assertEqual(extras[0][0], "moo")
-        self.assertEqual(extras[1][0], "moo_id")
 
 
 class TestLookupDimension(RecipeTestCase):
