@@ -4,16 +4,20 @@ Test recipes built from yaml files in the ingredients directory.
 """
 
 import os
+import time
+import warnings
+from copy import deepcopy
 from datetime import date
 
 from dateutil.relativedelta import relativedelta
+import yaml
 
 from recipe import AutomaticFilters, BadIngredient, InvalidIngredient, Shelf
 from tests.test_base import RecipeTestCase
 
 
 class ConfigTestBase(RecipeTestCase):
-    """A base class for testing shelves built from v1 or v2 config."""
+    """A base class for testing shelves built from config."""
 
     # The directory to look for yaml config files
     yaml_location = "shelf_config"
@@ -26,9 +30,9 @@ class ConfigTestBase(RecipeTestCase):
         contents = open(fn).read()
         return self.shelf_from_yaml(contents, selectable)
 
-    def shelf_from_yaml(self, yaml_config, selectable):
+    def shelf_from_yaml(self, yaml_config, selectable, **kwargs):
         """Create a shelf directly from configuration"""
-        return Shelf.from_validated_yaml(yaml_config, selectable)
+        return Shelf.from_validated_yaml(yaml_config, selectable, **kwargs)
 
 
 class TestNullHandling(ConfigTestBase):
@@ -978,17 +982,17 @@ ORDER BY state_raw""",
         self.assertRecipeCSV(
             recipe,
             """
-            state_idval_id,state_idval,avgage,state_idval_id
-            5033,Tennessee,84.0,5033
-            5562,Tennessee,83.0,5562
-            6452,Tennessee,82.0,6452
-            7322,Tennessee,81.0,7322
-            8598,Tennessee,80.0,8598
-            9583,Tennessee,79.0,9583
-            10501,Tennessee,84.0,10501
-            10672,Tennessee,78.0,10672
-            11141,Tennessee,83.0,11141
-            11168,Tennessee,77.0,11168
+            state_idval_id,state_idval,avgage
+            5033,Tennessee,84.0
+            5562,Tennessee,83.0
+            6452,Tennessee,82.0
+            7322,Tennessee,81.0
+            8598,Tennessee,80.0
+            9583,Tennessee,79.0
+            10501,Tennessee,84.0
+            10672,Tennessee,78.0
+            11141,Tennessee,83.0
+            11168,Tennessee,77.0
             """,
         )
 
@@ -1048,13 +1052,169 @@ ORDER BY state_raw""",
         )
 
 
+class TestMultipleSelectables(ConfigTestBase):
+    def test_multi_selectable(self):
+        """A shelf can reference multiple selectables"""
+        shelf_config = """
+username:
+  kind: Dimension
+  field: username
+department:
+  kind: Dimension
+  field: department
+testid:
+  kind: Dimension
+  field: testid
+score:
+  kind: Metric
+  field: avg(score)
+tagscorestest_cnt:
+  kind: Metric
+  field: count_distinct(tagscores.testid)
+tagscoresdepartment:
+  kind: Dimension
+  field: tagscores.department
+  filter: tagscores.username = username
+tagscorestestid:
+  kind: Dimension
+  field: tagscores.testid
+  filter: tagscores.username = username
+tagscoresscore:
+  kind: Metric
+  field: min(tagscores.score)
+  filter: tagscores.username = username
+        """
+
+        shelf = self.shelf_from_yaml(
+            shelf_config,
+            self.scores_with_nulls_table,
+            extra_selectables=[(self.tagscores_table, "tagscores")],
+        )
+
+        recipe = self.recipe(shelf=shelf).metrics("score").dimensions("department")
+        self.assertRecipeSQL(
+            recipe,
+            """
+            SELECT scores_with_nulls.department AS department,
+                avg(scores_with_nulls.score) AS score
+            FROM scores_with_nulls
+            GROUP BY department
+        """,
+        )
+        self.assertRecipeCSV(
+            recipe,
+            """
+            department,score,department_id
+            ,80.0,
+            ops,90.0,ops
+            sales,,sales
+            """,
+        )
+
+        # Now use multiple tables
+        # We are doing a cross join BUT the filter key on
+        # tagscorestestid allows us to join the two tables.
+        # Note: Shelves created from config will implicitly get a _select_from
+        # and will not raise an BadRecipe when selecting from multiple tables.
+        recipe = self.recipe(shelf=shelf).metrics("score").dimensions("tagscorestestid")
+        self.assertRecipeSQL(
+            recipe,
+            """
+SELECT tagscores.testid AS tagscorestestid,
+       avg(scores_with_nulls.score) AS score
+FROM tagscores,
+     scores_with_nulls
+WHERE tagscores.username = scores_with_nulls.username
+GROUP BY tagscorestestid
+        """,
+        )
+
+        # Now use multiple tables
+        # We are doing a cross join BUT the filter key on
+        # tagscorestestid allows us to join the two tables.
+        # Note: Shelves created from config will implicitly get a _select_from
+        # and will not raise an BadRecipe when selecting from multiple tables.
+        recipe = self.recipe(shelf=shelf).metrics("score").dimensions("tagscorestestid")
+        self.assertRecipeSQL(
+            recipe,
+            """
+SELECT tagscores.testid AS tagscorestestid,
+       avg(scores_with_nulls.score) AS score
+FROM tagscores,
+     scores_with_nulls
+WHERE tagscores.username = scores_with_nulls.username
+GROUP BY tagscorestestid
+        """,
+        )
+
+        # We can select multiple dimensions and metrics from both tables
+        # We will not duplicate the join filter as long as it has the same sql representation
+        recipe = (
+            self.recipe(shelf=shelf)
+            .metrics("score", "tagscoresscore")
+            .dimensions("tagscorestestid", "tagscoresdepartment")
+        )
+        self.assertRecipeSQL(
+            recipe,
+            """
+SELECT tagscores.department AS tagscoresdepartment,
+       tagscores.testid AS tagscorestestid,
+       avg(scores_with_nulls.score) AS score,
+       min(tagscores.score) AS tagscoresscore
+FROM tagscores,
+     scores_with_nulls
+WHERE tagscores.username = scores_with_nulls.username
+GROUP BY tagscoresdepartment,
+         tagscorestestid
+        """,
+        )
+        self.assertRecipeCSV(
+            recipe,
+            """
+tagscoresdepartment,tagscorestestid,score,tagscoresscore,tagscoresdepartment_id,tagscorestestid_id
+ops,2,90.0,80.0,ops,2
+ops,3,90.0,90.0,ops,3
+ops,4,90.0,100.0,ops,4
+ops,5,80.0,80.0,ops,5
+ops,6,80.0,90.0,ops,6
+sales,1,,80.0,sales,1
+            """,
+        )
+
+        # FIXME: If we try to only use the tagscores table, we get a crossjoin.
+        # It would be better if we only added the filter conditionally.
+        recipe = (
+            self.recipe(shelf=shelf)
+            .metrics("tagscoresscore")
+            .dimensions("tagscoresdepartment")
+        )
+        self.assertRecipeSQL(
+            recipe,
+            """
+SELECT tagscores.department AS tagscoresdepartment,
+       min(tagscores.score) AS tagscoresscore
+FROM tagscores,
+     scores_with_nulls
+WHERE tagscores.username = scores_with_nulls.username
+GROUP BY tagscoresdepartment
+        """,
+        )
+        self.assertRecipeCSV(
+            recipe,
+            """
+tagscoresdepartment,tagscoresscore,tagscoresdepartment_id
+ops,80.0,ops
+sales,80.0,sales
+            """,
+        )
+
+
 class TestParsedSQLGeneration(ConfigTestBase):
     """More tests of SQL generation on complex parsed expressions"""
 
     def test_weird_table_with_column_named_true(self):
         shelf = self.shelf_from_yaml(
             """
-_version: 2
 "true":
     kind: Dimension
     field: "[true]"
@@ -1071,11 +1231,42 @@ _version: 2
             """,
         )
 
+    def test_included_filter(self):
+        """Test ingredient definitions that include a filter expression"""
+        shelf = self.shelf_from_yaml(
+            """
+username:
+    kind: Dimension
+    field: username
+    filter: username = "foo"
+""",
+            self.scores_with_nulls_table,
+        )
+        recipe = self.recipe(shelf=shelf).dimensions("username")
+        self.assertRecipeSQL(
+            recipe,
+            """SELECT scores_with_nulls.username AS username
+FROM scores_with_nulls
+WHERE scores_with_nulls.username = 'foo'
+GROUP BY username""",
+        )
+
+        # Filters that aren't datatype=bool raise errors.
+        with self.assertRaises(BadIngredient):
+            shelf = self.shelf_from_yaml(
+                """
+    username:
+        kind: Dimension
+        field: username
+        filter: username + "moo"
+    """,
+                self.scores_with_nulls_table,
+            )
+
     def test_complex_field(self):
         """Test parsed field definitions that use math, field references and more"""
         shelf = self.shelf_from_yaml(
             """
-_version: 2
 username:
     kind: Dimension
     field: username
@@ -1297,7 +1488,6 @@ FROM scores_with_nulls""",
         """Test parsed field definitions built on top of other selectables"""
         shelf = self.shelf_from_yaml(
             """
-_version: 2
 username:
     kind: Dimension
     field: username
@@ -1328,7 +1518,6 @@ count_star:
         # Build a recipe using the first recipe
         shelf2 = self.shelf_from_yaml(
             """
-_version: 2
 count_star:
     kind: Metric
     field: "count(*)"
@@ -1352,6 +1541,230 @@ count_username:
         self.assertRecipeCSV(recipe2, "count_star,count_username\n3,3\n")
 
 
+class TestShelfConstants(ConfigTestBase):
+    def test_simple_constant(self):
+        cache = Cache()
+        shelf = self.shelf_from_yaml(
+            """
+            username: {kind: Dimension, field: username+constants.twostr}
+            count_star: {kind: Metric, field: count(*) * constants.sumscore}
+            count_star_times_two: {kind: Metric, field: constants.two*count(*)}
+            convertdate: {kind: Dimension, field: month(test_date)}
+            """,
+            self.scores_with_nulls_table,
+            ingredient_cache=cache,
+            constants={
+                "two": 2,
+                "twofloat": 2.0,
+                "twostr": "two",
+                "sumscore": "sum(score)",
+            },
+        )
+        recipe = (
+            self.recipe(shelf=shelf)
+            .metrics("count_star", "count_star_times_two")
+            .dimensions("username")
+        )
+        self.assertRecipeSQL(
+            recipe,
+            """SELECT scores_with_nulls.username || CAST('two' AS VARCHAR) AS username,
+       count(*) * constants.sumscore AS count_star,
+       CAST(2 AS INTEGER) * count(*) AS count_star_times_two
+FROM scores_with_nulls,
+
+  (SELECT sum(scores_with_nulls.score) AS sumscore
+   FROM scores_with_nulls) AS constants
+GROUP BY username""",
+        )
+        self.assertRecipeCSV(
+            recipe,
+            """username,count_star,count_star_times_two,username_id
+annikatwo,520.0,4,annikatwo
+chiptwo,780.0,6,chiptwo
+christwo,260.0,2,christwo""",
+        )
+
+    def test_census_constants(self):
+        cache = Cache()
+        shelf = self.shelf_from_yaml(
+            """
+            state: {kind: Dimension, field: state}
+            pop2000: {kind: Metric, field: sum(pop2000)}
+            pop2000_of_total: {kind: Metric, field: sum(pop2000)/constants.ttlpop}
+            """,
+            self.census_table,
+            ingredient_cache=cache,
+            constants={"ttlpop": "sum(pop2000)"},
+        )
+        recipe = self.recipe(shelf=shelf).metrics("pop2000").dimensions("state")
+        self.assertRecipeSQL(
+            recipe,
+            """SELECT census.state AS state,
+       sum(census.pop2000) AS pop2000
+FROM census
+GROUP BY state""",
+        )
+        self.assertRecipeCSV(
+            recipe,
+            """state,pop2000,state_id
+Tennessee,5685230,Tennessee
+Vermont,609480,Vermont""",
+        )
+        recipe = (
+            self.recipe(shelf=shelf)
+            .metrics("pop2000", "pop2000_of_total")
+            .dimensions("state")
+        )
+        self.assertRecipeSQL(
+            recipe,
+            """SELECT census.state AS state,
+       sum(census.pop2000) AS pop2000,
+       CASE
+           WHEN (constants.ttlpop = 0) THEN NULL
+           ELSE CAST(sum(census.pop2000) AS FLOAT) / CAST(constants.ttlpop AS FLOAT)
+       END AS pop2000_of_total
+FROM census,
+
+  (SELECT sum(census.pop2000) AS ttlpop
+   FROM census) AS constants
+GROUP BY state""",
+        )
+        self.assertRecipeCSV(
+            recipe,
+            """state,pop2000,pop2000_of_total,state_id
+Tennessee,5685230,0.9031758413016644,Tennessee
+Vermont,609480,0.09682415869833559,Vermont""",
+        )
+
+
+class Cache(dict):
+    def set(self, k, v):
+        self[k] = v
+
+
+class TestCache(ConfigTestBase):
+    def test_cache(self):
+        cache = Cache()
+        self.shelf_from_yaml(
+            """
+            username: {kind: Dimension, field: username}
+            count_star: {kind: Metric, field: count(*)}
+            convertdate: {kind: Dimension, field: month(test_date)}
+            """,
+            self.scores_with_nulls_table,
+            ingredient_cache=cache,
+        )
+        self.assertEqual(len(cache), 1)
+        ingredients = cache[list(cache.keys())[0]]
+        self.assertEqual(len(ingredients), 3)
+
+    def test_selectables_cache(self):
+        """Test cache when the selectable is a recipe"""
+        cache = Cache()
+        shelf = self.shelf_from_yaml(
+            """
+            username: {kind: Dimension, field: username}
+            count_star: {kind: Metric, field: 'count(*)'}
+            """,
+            self.scores_with_nulls_table,
+            ingredient_cache=cache,
+        )
+        recipe = self.recipe(shelf=shelf).dimensions("username").metrics("count_star")
+
+        self.assertEqual(len(cache), 1)
+        first_cache_key = list(cache.keys())[0]
+        self.assertEqual(len(cache[first_cache_key]), 2)
+
+        # Build a recipe using the first recipe
+        self.shelf_from_yaml(
+            "count_username: {kind: Metric, field: 'count(username)'}",
+            recipe,
+            ingredient_cache=cache,
+        )
+
+        self.assertEqual(len(cache), 2)
+        second_cache_key = list(cache.keys() - {first_cache_key})[0]
+        self.assertEqual(len(cache[second_cache_key]), 1)
+
+    def test_cache_is_faster(self):
+        yml = """
+        username: {kind: Dimension, field: username}
+        count_star: {kind: Metric, field: "count(*)"}
+        convertdate: {kind: Dimension, field: "month(test_date)"}
+        strings: {kind: Dimension, field: "string(test_date)+string(score)"}
+        total_nulls: {kind: Metric, field: "count_distinct(if(score IS NULL, username))"}
+        chip_nulls: {kind: Metric, field: 'sum(if(score IS NULL and username = "chip",1,0))'}
+        user_null_counter: {kind: Metric, field: 'if(username IS NULL, 1, 0)'}
+        chip_or_nulls: {kind: Metric, field: 'sum(if(score IS NULL OR (username = "chip"),1,0))'}
+        simple_math: {kind: Metric, field: "@count_star +  @total_nulls   + @chip_nulls"}
+        refs_division: {kind: Metric, field: "@count_star / 100.0"}
+        refs_as_denom: {kind: Metric, field: "12 / @count_star"}
+        math: {kind: Metric, field: "(@count_star / @count_star) + (5.0 / 2.0)"}
+        parentheses: {kind: Metric, field: "@count_star / (@count_star + (12.0 / 2.0))"}
+        """
+        config = yaml.safe_load(yml)
+        uncached_start = time.time()
+        COUNT = 2
+        for i in range(COUNT):
+            Shelf.from_config(deepcopy(config), self.scores_with_nulls_table)
+        uncached_duration = time.time() - uncached_start
+
+        cache = Cache()
+        # prime the cache
+        Shelf.from_config(
+            deepcopy(config), self.scores_with_nulls_table, ingredient_cache=cache
+        )
+        cached_start = time.time()
+        for i in range(COUNT):
+            Shelf.from_config(
+                deepcopy(config), self.scores_with_nulls_table, ingredient_cache=cache
+            )
+        cached_duration = time.time() - cached_start
+
+        # usually this performance is somewhere between 100 and 1000 times faster, but
+        # we should be conservative here
+        if not cached_duration < (uncached_duration / 50):
+            # let's just warn instead of actually failing the test suite
+            warnings.warn(
+                "cache was not fast enough: "
+                f"cached duration {cached_duration}, "
+                f"uncached duration: {uncached_duration}",
+                UserWarning,
+            )
+
+    def test_broken_cache(self):
+        """If the cache has corrupt data, it is ignored"""
+        cache = Cache()
+        self.shelf_from_yaml(
+            """
+            username: {kind: Dimension, field: username}
+            count_star: {kind: Metric, field: count(*)}
+            convertdate: {kind: Dimension, field: month(test_date)}
+            """,
+            self.scores_with_nulls_table,
+            ingredient_cache=cache,
+        )
+        og_cache = deepcopy(cache)
+        self.assertEqual(len(cache), 1)
+        main_cache_key = list(cache.keys())[0]
+        broken_ingredients = {
+            k: ({"broken": "tree"}, {"broken": "validator"})
+            for k in cache[main_cache_key]
+        }
+        cache[main_cache_key] = broken_ingredients
+        self.shelf_from_yaml(
+            """
+            username: {kind: Dimension, field: username}
+            count_star: {kind: Metric, field: count(*)}
+            convertdate: {kind: Dimension, field: month(test_date)}
+            """,
+            self.scores_with_nulls_table,
+            ingredient_cache=cache,
+        )
+        # the cache should be reinitialized, and should be identical to the old cache
+        self.assertEqual(cache, og_cache)
+
+
 class TestParsedIntellligentDates(ConfigTestBase):
     def test_is_current_year(self):
         """Test current year with a variety of spacing and capitalization"""
@@ -1366,7 +1779,6 @@ class TestParsedIntellligentDates(ConfigTestBase):
         for is_current_year in data:
             shelf = self.shelf_from_yaml(
                 """
-_version: 2
 test:
     kind: Metric
     field: "if(dt {}, count, 0)"
@@ -1397,7 +1809,6 @@ test:
         for is_prior_year in data:
             shelf = self.shelf_from_yaml(
                 f"""
-_version: 2
 test:
     kind: Metric
     field: "if(dt {is_prior_year}, count, 0)"
@@ -1435,7 +1846,6 @@ test:
         for ytd in data:
             shelf = self.shelf_from_yaml(
                 f"""
-_version: 2
 test:
     kind: Metric
     field: "if(dt {ytd}, count, 0)"
@@ -1464,7 +1874,6 @@ test:
         for ytd in data:
             shelf = self.shelf_from_yaml(
                 f"""
-_version: 2
 test:
     kind: Metric
     field: "if(not(dt {ytd}), count, 0)"
@@ -1486,7 +1895,6 @@ test:
         for ytd in data:
             shelf = self.shelf_from_yaml(
                 f"""
-_version: 2
 test:
     kind: Metric
     field: "if(dt {ytd}, count, 0)"
@@ -1499,11 +1907,11 @@ test:
         assert len(unique_sql) == 3
 
     def test_convert_date(self):
-        """We can convert dates using formats"""
+        """We can convert dates using formats.
+        But it is better to use the date_aggregation property"""
 
         shelf = self.shelf_from_yaml(
             """
-_version: 2
 test:
     kind: Dimension
     field: dt
@@ -1545,6 +1953,74 @@ test5:
                     test5""",
         )
 
+    def test_convert_date_with_date_aggregation(self):
+        """We can convert dates using date_aggregation"""
+
+        shelf = self.shelf_from_yaml(
+            """
+test:
+    kind: Dimension
+    field: dt
+    date_aggregation: year
+test2:
+    kind: Dimension
+    field: dt
+    date_aggregation: year
+test3:
+    kind: Dimension
+    field: dt
+    date_aggregation: month
+test4:
+    kind: Dimension
+    field: dt
+    date_aggregation: month
+test5:
+    kind: Dimension
+    field: dt
+    format: ".2f"
+""",
+            self.datetester_table,
+        )
+        recipe = self.recipe(shelf=shelf).dimensions(
+            "test", "test2", "test3", "test4", "test5"
+        )
+        self.assertRecipeSQL(
+            recipe,
+            """SELECT date_trunc('year', datetester.dt) AS test,
+                date_trunc('year', datetester.dt) AS test2,
+                date_trunc('month', datetester.dt) AS test3,
+                date_trunc('month', datetester.dt) AS test4,
+                datetester.dt AS test5
+            FROM datetester
+            GROUP BY test,
+                    test2,
+                    test3,
+                    test4,
+                    test5""",
+        )
+
+    def test_invalid_date_aggregation(self):
+        """Invalid date formats are ignored in a shelf construction
+        and would fail ingredient validation."""
+
+        shelf = self.shelf_from_yaml(
+            """
+test:
+    kind: Dimension
+    field: dt
+    date_aggregation: years
+""",
+            self.datetester_table,
+        )
+        recipe = self.recipe(shelf=shelf).dimensions("test")
+        # No date aggregation is applied.
+        self.assertRecipeSQL(
+            recipe,
+            """SELECT datetester.dt AS test
+FROM datetester
+GROUP BY test""",
+        )
+
     def test_id_fields(self):
         """We can have fields that end in _id
 
@@ -1553,7 +2029,6 @@ test5:
 
         shelf = self.shelf_from_yaml(
             """
-_version: 2
 student:
     kind: Dimension
     field: student
@@ -1624,7 +2099,6 @@ class TestParsedFieldConfig(ConfigTestBase):
         """Test the trees generated by parsing fields"""
         shelf = self.shelf_from_yaml(
             """
-_version: 2
 convertdate:
     kind: Dimension
     field: "month(test_date)"
